#
# Mooshak: managing programming contests on the web		May 2004
# 
#			Zé Paulo Leal 		
#			zp@ncc.up.pt
#
#-----------------------------------------------------------------------------
# file: Sessions.tcl
# 
## Set of user sessions 

package provide Sessions 1.0

package require data
package require Session

namespace eval Sessions {

    variable Default_Timeout	3600	;## default timeout, in seconds
     variable Tip

    array set Tip {
	Timeout		"Maximum time in seconds for an idle session"
    }
}

Attributes Sessions {

    Timeout text	{}
    Session dirs	Session
    
}

Operation Sessions::_create_ {} {
    ${_Self_} _update_
}

Operation Sessions::_update_ {} {

    if { ! [ regexp {^[0-9]+$} $Timeout ] } {
	variable Default_Timeout

	set Timeout $Default_Timeout
    }
    return 0
}


## Return number of currently active sessions 
Operation Sessions::count {} {
    
    return [ llength [ glob -nocomplain -type d ${_Self_}/* ] ]
}

## Cleanup sessions and remove inactive sessions.
## Sessions modified more than $Timeout seconds ago are deleted.
Operation Sessions::cleanup {} {
    set now [ clock seconds ]

    foreach session [ glob -nocomplain -type d ${_Self_}/* ] {
	if { [ file mtime $session ] + $Timeout < $now } {
	    file delete -force -- $session
	}
    }
}

## Notify users with messge. Messages will be shown in an alert box.
## User name and profile can be used to select message recipients.
## By default messages are delivered to all users with team profile.
Operation Sessions::notify {message {user *} {profile team} } {
    variable ::Session::Lock

    set pattern [ format %s/*_%s_%s ${_Self_} $user $profile ]

    foreach session [ glob -nocomplain -type d $pattern ] {

	## USE THIS LOCK WHEN WRITTING SESSIONS!!!
	file::lock $session
	set sp [ data::open $session ]
	lappend ${sp}::messages $message 
	data::record $session
	file::unlock $session
    }

}
